
'''
Script to plot summary snowpit from data example of the standard snowpit format
Simon Filhol, December 2016

'''

from snowpyt import pit_class as pc

filename = '/home/arcticsnow/github/snowpyt/snowpyt/data_example/20170209_Finse_snowpit.xlsx'  #[insert path to file]

pit1 = Snowpit()
pit1.filename = filename
pit1.import_xlsx()

pit1.plot(metadata=True)
pit1.plot(plots_order=['density', 'temperature', 'stratigraphy','crystal size'])







# -*- coding: utf-8 -*-
"""
Created on Tue Jul 04 13:32:31 2017

@author: Guillaume Sutter

Collection of functions to import snowpit data stored in the CAAML xml standard
    
"""

import xml.dom.minidom
import pit_class as pc

def get_metadata(path_xml):
    DOMTree = xml.dom.minidom.parse(path_xml)
    Tree = DOMTree.documentElement
    Metadata = pc.metadata()

    Metadata.date = Tree.getElementsByTagName("timePosition")[0].firstChild.nodeValue
    Metadata.operation = Tree.getElementsByTagName("Operation")[0].childNodes[1].firstChild.nodeValue
    Metadata.observer = Tree.getElementsByTagName("Operation")[0].childNodes[3].childNodes[1].childNodes[
        1].firstChild.nodeValue
    Metadata.profile_depth = float(Tree.getElementsByTagName("profileDepth")[0].firstChild.nodeValue)
    Metadata.profile_depth_unit = Tree.getElementsByTagName("profileDepth")[0].attributes.item(0).value
    Metadata.sky_condition = Tree.getElementsByTagName("skyCond")[0].firstChild.nodeValue
    Metadata.precipitation = Tree.getElementsByTagName("precipTI")[0].firstChild.nodeValue
    Metadata.air_temperature = float(Tree.getElementsByTagName("airTempPres")[0].firstChild.nodeValue)
    Metadata.air_temperature_unit = Tree.getElementsByTagName("airTempPres")[0].attributes.item(0).value
    Metadata.windspeed = float(Tree.getElementsByTagName("windSpd")[0].firstChild.nodeValue)
    Metadata.windspeed_unit = Tree.getElementsByTagName("windSpd")[0].attributes.item(0).value
    Metadata.elevation = float(Tree.getElementsByTagName("ElevationPosition")[0].childNodes[1].firstChild.nodeValue)
    Metadata.elevation_unit = Tree.getElementsByTagName("ElevationPosition")[0].attributes.item(0).value
    Metadata.location_description = Tree.getElementsByTagName("ObsPoint")[0].childNodes[1].firstChild.nodeValue
    Metadata.srsName = Tree.getElementsByTagName("gml:Point")[0].attributes.item(0).value
    if Tree.getElementsByTagName("gml:Point")[0].childNodes[1].firstChild is not None:
        Metadata.east = float(
            Tree.getElementsByTagName("gml:Point")[0].childNodes[1].firstChild.nodeValue.split(" ")[0])
        Metadata.north = float(
            Tree.getElementsByTagName("gml:Point")[0].childNodes[1].firstChild.nodeValue.split(" ")[1])
    return Metadata

def get_layers(path_xml):
    DOMTree = xml.dom.minidom.parse(path_xml)
    Tree = DOMTree.documentElement
    stratProfile = Tree.getElementsByTagName("stratProfile")
    Layers = []
    for Layer in stratProfile[0].childNodes:
        if Layer.nodeType not in {3, 8}:
            lay = pc.layer()
            for child in Layer.childNodes:
                if child.nodeType not in {3, 8}:
                    if child.localName == "depthTop" and child.firstChild is not None:
                        lay.dtop = float(child.firstChild.nodeValue)
                        lay.dtop_unit = child.attributes.item(0).value
                    elif child.localName == "thickness" and child.firstChild is not None:
                        lay.thickness = float(child.firstChild.nodeValue)
                        lay.thickness_unit = child.attributes.item(0).value
                    elif child.localName == "grainFormPrimary" and child.firstChild is not None:
                        lay.grain_type1 = child.firstChild.nodeValue
                    elif child.localName == "grainFormSecondary" and child.firstChild is not None:
                        lay.grain_type2 = child.firstChild.nodeValue
                    elif child.localName == "grainSize":
                        lay.grain_size_unit = child.attributes.item(0).value
                        for component in child.childNodes[1].childNodes:
                            if component.nodeType not in {3, 8}:
                                if component.localName == "avg" and component.firstChild is not None:
                                    lay.grain_size_avg = float(component.firstChild.nodeValue)
                                elif component.localName == "avgMax" and component.firstChild is not None:
                                    lay.grain_size_max = float(component.firstChild.nodeValue)
                    elif child.localName == "hardness" and child.firstChild is not None:
                        lay.hardness = child.firstChild.nodeValue
                    elif child.localName == "lwc" and child.firstChild is not None:
                        lay.lwc = child.firstChild.nodeValue
            Layers += [lay]
    return Layers

def get_temperature(path_xml):
    DOMTree = xml.dom.minidom.parse(path_xml)
    Tree = DOMTree.documentElement
    temperatureProfile = Tree.getElementsByTagName("tempProfile")
    TProfile = pc.temperature_profile()
    TProfile.depth_unit = temperatureProfile[0].attributes.item(0).value
    TProfile.temp_unit = temperatureProfile[0].attributes.item(1).value
    for obs in temperatureProfile[0].childNodes:
        if obs.nodeType not in {3, 8}:
            for child in obs.childNodes:
                if child.nodeType not in {3, 8}:
                    if child.localName == "depth" and child.firstChild is not None:
                        TProfile.depth += [float(child.firstChild.nodeValue)]
                    elif child.localName == "snowTemp" and child.firstChild is not None:
                        TProfile.temp += [float(child.firstChild.nodeValue)]
    return TProfile

def get_density(path_xml):
    DOMTree = xml.dom.minidom.parse(path_xml)
    Tree = DOMTree.documentElement
    densityProfile = Tree.getElementsByTagName("densityProfile")
    DProfile = pc.density_profile()
    DProfile.depth_unit = densityProfile[0].attributes.item(0).value
    DProfile.thickness_unit = densityProfile[0].attributes.item(1).value
    DProfile.density_unit = densityProfile[0].attributes.item(2).value
    for layer in densityProfile[0].childNodes:
        if layer.nodeType not in {3, 8}:
            for child in layer.childNodes:
                if child.nodeType not in {3, 8}:
                    if child.localName == "depthTop" and child.firstChild is not None:
                        DProfile.depth += [float(child.firstChild.nodeValue)]
                    elif child.localName == "thickness" and child.firstChild is not None:
                        DProfile.thickness += [float(child.firstChild.nodeValue)]
                    elif child.localName == "density" and child.firstChild is not None:
                        DProfile.density += [float(child.firstChild.nodeValue)]
    return DProfile

# if __main__():
#     # Example on how to use the functions
#     path1 = "snowpyt/data_example/20160331_finse.xml"
#     A = get_layers(path1)
#     B = get_temperature(path1)
#     C = get_density(path1)
#     D = get_metadata(path1)





# ==============================================================================
# for child in Tree.childNodes:
#     if child.nodeType not in {3,8}:
#         print "\n"+child.localName
#         for child2 in child.childNodes:
#             if child2.nodeType not in {3,8} and str(child2.firstChild)!="None":
#                 if child2.hasAttributes():
#                     if child2.firstChild.nodeType not in {3,8}:
#                         print "\t{}: {} {}".format(child2.localName,child2.firstChild.nodeValue,child2.attributes.item(0).value)
#                     else:
#                         print "\t{}: {}".format(child2.localName,child2.attributes.item(0).value)
#                 else:
#                     if str(child2.firstChild.nodeValue).split(" ")[0]!="\n":
#                         print "\t{}: {}".format(child2.localName,child2.firstChild.nodeValue)
#                     else:
#                         print "\t{}".format(child2.localName)
#                 for child3 in child2.childNodes:
#                     if child3.nodeType not in {3,8} and str(child3.firstChild)!="None":
#                         if child3.hasAttributes():
#                             if child3.firstChild.nodeType not in {3,8}:
#                                 print "\t\t{}: {} {}".format(child3.localName,child3.firstChild.nodeValue,child3.attributes.item(0).value)
#                             else:
#                                 print "\t\t{}: {}".format(child3.localName,child3.attributes.item(0).value)
#                         else:
#                             if str(child3.firstChild.nodeValue).split(" ")[0]!="\n":
#                                 print "\t\t{}: {}".format(child3.localName,child3.firstChild.nodeValue)
#                             else:
#                                 print "\t\t{}".format(child3.localName)
#                         for child4 in child3.childNodes:
#                             if child4.nodeType not in {3,8} and str(child4.firstChild)!="None":
#                                 if child4.hasAttributes():
#                                     if child4.firstChild.nodeType not in {3,8}:
#                                         print "\t\t\t{}: {} {}".format(child4.localName,child4.firstChild.nodeValue,child4.attributes.item(0).value)
#                                     else:
#                                         print "\t\t\t{}: {}".format(child4.localName,child4.attributes.item(0).value)
#                                 else:
#                                     if str(child4.firstChild.nodeValue).split(" ")[0]!="\n":
#                                         print "\t\t\t{}: {}".format(child4.localName,child4.firstChild.nodeValue)
#                                     else:
#                                         print "\t\t\t{}".format(child4.localName)
#                                 for child5 in child4.childNodes:
#                                     if child5.nodeType not in {3,8} and str(child5.firstChild)!="None":
#                                         if child5.hasAttributes():
#                                             if child5.firstChild.nodeType not in {3,8}:
#                                                 print "\t\t\t\t{}: {} {}".format(child5.localName,child5.firstChild.nodeValue,child5.attributes.item(0).value)
#                                             else:
#                                                 print "\t\t\t\t{}: {}".format(child5.localName,child5.attributes.item(0).value)
#                                         else:
#                                             if str(child5.firstChild.nodeValue).split(" ")[0]!="\n":
#                                                 print "\t\t\t\t{}: {}".format(child5.localName,child5.firstChild.nodeValue)
#                                             else:
#                                                 print "\t\t\t\t{}".format(child5.localName)
#                                         for child6 in child5.childNodes:
#                                             if child6.nodeType not in {3,8} and str(child6.firstChild)!="\n":
#                                                 if child6.hasAttributes():
#                                                     if child6.firstChild.nodeType not in {3,8}:
#                                                         print "\t\t\t\t\t{}: {} {}".format(child6.localName,child6.firstChild.nodeValue,child6.attributes.item(0).value)
#                                                     else:
#                                                         print "\t\t\t\t\t{}: {}".format(child6.localName,child6.attributes.item(0).value)
#                                                 else:
#                                                     if str(child6.firstChild.nodeValue).split(" ")[0]!="\n":
#                                                         print "\t\t\t\t\t{}: {}".format(child6.localName,child6.firstChild.nodeValue)
#                                                     else:
#                                                         print "\t\t\t\t\t{}".format(child6.localName)
#                                                 for child7 in child6.childNodes:
#                                                     if child7.nodeType not in {3,8} and str(child7.firstChild)!="None":
#                                                         if child7.hasAttributes():
#                                                             if child7.firstChild.nodeType not in {3,8}:
#                                                                 print "\t\t\t\t\t\t{}: {} {}".format(child7.localName,child7.firstChild.nodeValue,child7.attributes.item(0).value)
#                                                             else:
#                                                                 print "\t\t\t\t\t\t{}: {}".format(child7.localName,child7.attributes.item(0).value)
#                                                         else:
#                                                             if str(child7.firstChild.nodeValue).split(" ")[0]!="\n":
#                                                                 print "\t\t\t\t\t\t{}: {}".format(child7.localName,child7.firstChild.nodeValue)
#                                                             else:
#                                                                 print "\t\t\t\t\t\t{}".format(child7.localName)
# ==============================================================================



from __future__ import division
import os, sys

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

# Include class and function here


class Snowpit_svalbard_JC(Snowpit):
    '''
    Class for snowpit data as sorted by JC about Snow Svalbard Research
    '''

    def __init__(self, filename):
        super(Snowpit, self).__init__()
        self.filename = 'example.txt'

    def summary_plot(self, save=False):
        '''
        plot general snowpit plot
        :return:
        '''

        fig = plt.figure(figsize=(10, 10), dpi=150)
        # fig = plt.figure()
        ax1 = plt.subplot2grid((4,4),(0,0),rowspan = 3)
        ax2 = plt.subplot2grid((4,4),(0,1),rowspan = 3, sharey=ax1)  # Share y-axes with subplot 1
        ax3 = plt.subplot2grid((4,4),(0,2),rowspan = 3, sharey=ax2)
        ax4 = plt.subplot2grid((4,4),(0,3),rowspan = 3, sharey=ax3)

        # Set y-ticks of subplot 2 invisible
        plt.setp(ax2.get_yticklabels(), visible=False)
        plt.setp(ax2.get_xticklabels(), visible=False)
        plt.setp(ax3.get_yticklabels(), visible=False)
        plt.setp(ax4.get_yticklabels(), visible=False)

        # Plot data
        fig.gca().invert_yaxis()
        im1 = ax1.plot(-self.temperature_snow, self.temperature_depth)

        im2 = ax2.barh(self.layer_top, np.repeat(1, self.layer_top.__len__()), self.layer_bot - self.layer_top,
                       color=cm.Blues(self.hardness_code / 7))
        ax2.set_xlim(0, 1)

        # include symbols

        for i, flake in enumerate(self.grain_type1.astype(str)):
            if flake != 'nan':
                im = plt.imread(snowflake_dict.get(flake))
                im[im == 0] = np.nan
                imagebox = OffsetImage(im, zoom=.02)
                if self.grain_type2.astype(str)[i] == 'nan':
                    hloc = 0.5
                else:
                    hloc = 0.33
                xy = [hloc,
                      ((self.layer_top[i] - self.layer_bot[i]) / 2 + self.layer_bot[i])]  # coordinates to position this image
                ab = AnnotationBbox(imagebox, xy, xycoords='data', boxcoords='data', frameon=False)
                ax2.add_artist(ab)

        for i, flake in enumerate(self.grain_type2.astype(str)):
            if flake != 'nan':
                im = plt.imread(snowflake_dict.get(flake))
                im[im == 0] = np.nan
                imagebox = OffsetImage(im, zoom=.02)
                xy = [0.66,
                      ((self.layer_top[i] - self.layer_bot[i]) / 2 + self.layer_bot[i])]  # coordinates to position this image
                ab = AnnotationBbox(imagebox, xy, xycoords='data', boxcoords='data', frameon=False)
                ax2.add_artist(ab)

        im3 = ax3.barh(self.layer_top, self.hardness_code, self.layer_bot - self.layer_top, color=cm.Blues(self.hardness_code / 6))
        ax3.set_xlim(0, 7)

        im4 = ax4.plot(self.density, self.density_depth)
        ax4.yaxis.tick_right()

        # add
        ax1.set_title("Temperature ($^\circ$C)")
        ax2.set_title("Stratigraphy")
        ax3.set_title("Hardness")
        ax4.set_title("Density")
        ax1.set_ylabel("Depth (cm)")

        ax1.grid()
        ax4.grid()

        metadata = "Date: " + self.date + '\n' + \
                   "Observer: " + self.Observer + '\n' + \
                   "Glacier: " + self.glacier + '\n' + \
                   "East : " + self.East + '\n' + \
                   "North: " + self.North + '\n' + \
                   "Elevation: " + self.Elevation + '\n' + \
                   "Weather Conditions: " + self.weather_conditions + '\n' + \
                   "Air temperature: " + self.AirTemp + '\n' + \
                   "Comments: " + self.comments + '\n'
        left, width = .25, .5
        bottom, height = .25, .5
        right = left + width
        top = bottom + height

        plt.figtext(0.08, 0.15 , metadata,
                    horizontalalignment='left',
                    verticalalignment='center',wrap=True, fontsize=6)
        #fig.autofmt_xdate()

        plt.tight_layout()
        plt.subplots_adjust(wspace=0)
        if save==True:
            fig.savefig(self.filename.split('/')[-1][0:-4])


    def plot_temperature(self):
        '''
        Plot temperature profile

        TODO:
            - reverse depth axis
        '''
        plt.figure()
        plt.plot(self.temperature_snow, self.temperature_depth)
        plt.gca().invert_yaxis()
        plt.xlabel('Temperature (C)')
        plt.ylabel('Depth (cm)')
        plt.title('Temperature profile')
        plt.grid()

    def plot_density(self):
        '''
        Plot density profile

        TODO:
            - reverse depth axis
        '''
        plt.figure()
        plt.plot(self.density, self.density_depth)
        plt.xlabel('Density (kg/m3)')
        plt.ylabel('Depth (cm)')
        plt.title('Density profile')
        plt.grid()

    def load_csv(self):
        self.load_metadata()
        self.load_profile()

    def load_profile(self):
        self.profile_raw_table = pd.read_csv(self.filename, sep='\t', skiprows=14)
        self.layerID = self.profile_raw_table['Layer']
        self.layer_top = self.profile_raw_table['Top [cm]']
        self.layer_bot = self.profile_raw_table['Bottom [cm]']

        self.grain_type1 = self.profile_raw_table['Type 1']
        self.grain_type2 = self.profile_raw_table['Type 2']
        self.grain_type2 = self.profile_raw_table['Type 3']
        self.grain_size_min = self.profile_raw_table['Diameter min [mm]']
        self.grain_size_max = self.profile_raw_table['Diameter max [mm]']

        self.hardness = self.profile_raw_table['Hardness']
        self.hardness_code = self.profile_raw_table['Hardness code']

        self.density_depth = self.profile_raw_table['Depth Center [cm]']
        self.density = self.profile_raw_table['Snow Density [g/cm3]']

        self.temperature_depth = self.profile_raw_table['Depth [cm]']
        self.temperature_snow = self.profile_raw_table['Temp [deg C]']

        self.depth_sample = self.profile_raw_table['Depth Center [cm].1']
        self.name_sample = self.profile_raw_table['ID_sample']



    def load_metadata(self):
        f = open(self.filename)
        try:
            for i, line in enumerate(f):
                if line[0:4] == 'Date':
                    self.date = line.split("\t")[1]
                    self.East = line.split("\t")[6]
                if line[0:4] == 'Time':
                    self.Elevation = line.split("\t")[5]
                if line[0:4] == 'Area':
                    self.North = line.split("\t")[7]
                if line[0:4] == 'Obse':
                    self.Observer = line.split("\t")[1]
                if line[0:4] == 'Air ':
                    self.AirTemp = line.split("\t")[2]
                if line[0:4] == 'Glac':
                    self.glacier = line.split("\t")[1]
                if line[0:4] == 'Glac':
                    self.weather_conditions = line.split("\t")[4]
                if line[0:4] == 'Gene':
                    self.comments = line.split("\t")[1]
        except ValueError:
            print "Could not load metadata. Check file formating"
        f.close()

    def print_metadata(self):
        print "Date: " + self.date
        print "East [deg]: " + self.East
        print "North [deg]: " + self.North
        print "Elevation [m]: " + self.Elevation
        print "Observer: " + self.Observer
        print "Air temperature [C]: " + self.AirTemp
        print "Glacier: " + self.glacier
        print "Weather conditions: " + self.weather_conditions
        print "Comments: " + self.comments




# Include script in this if statement
if __name__ == '__main__':